package weaver.interfaces.hostar.job;

import cn.hutool.core.date.DateUtil;
import com.engine.hostar.thread.HandleCBDataThread;
import com.engine.hostar.util.HostarUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.ThreadPoolUtil;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;

/**
 * @version 1.0
 * @Title ecology-9
 * @Company 泛微软件
 * @CreateDate 2024/3/12
 * @Description 更新餐补信息记录数据定时任务
 * @Author AdminZm
 */
public class UpdateCbxxjlDataJob extends BaseCronJob {

    BaseBean baseBean = new BaseBean();

    private String ryParam;

    private String startDate;

    private String endDate;

    @Override
    public void execute() {
        baseBean.writeLog("UpdateCbxxjlDataJob start, param is:" + ryParam + "、" + startDate + "、" + endDate);
        try {
            String today = DateUtil.format(new Date(), "yyyy-MM-dd");
            if (StringUtils.isEmpty(endDate)) {
                endDate = today;
            }
            if (StringUtils.isEmpty(startDate)) {
                startDate = today;
            }
            String date1 = String.valueOf(startDate);
            String date2 = String.valueOf(endDate);
            List<String> allDates = getAllDates(date1, date2);
            baseBean.writeLog("UpdateCbxxjlDataJob allDatas:" + allDates);
            RecordSet rs = new RecordSet();
            List<String> userIds = new ArrayList<>();
            if (StringUtils.isEmpty(ryParam) || StringUtils.equals(ryParam, "null")) {
                baseBean.writeLog("1111.");
                rs.executeQuery("select id from hrmresource");
                while (rs.next()) {
                    userIds.add(rs.getString("id"));
                }
            } else {
                baseBean.writeLog("2222.");
                rs.executeQuery("select id from hrmresource where workcode = ?", ryParam);
                while (rs.next()) {
                    userIds.add(rs.getString("id"));
                }
            }
            baseBean.writeLog("userIds:" + userIds);
            if ((CollectionUtils.isEmpty(userIds) && userIds.isEmpty()) || (CollectionUtils.isEmpty(allDates) && allDates.isEmpty())) {
                baseBean.writeLog("no user or date.");
                return;
            }
            ExecutorService executorService = ThreadPoolUtil.getThreadPool(null, null);
            for (String userId : userIds) {
                for (String kqDate : allDates) {
                    executorService.execute(new HandleCBDataThread(userId, kqDate));
                }
            }
            if (executorService.isTerminated()) {
                executorService.shutdown();
            }
        } catch (Exception e) {
            baseBean.writeLog("UpdateCbxxjlDataJob error:" + e.getMessage());
        }
    }

    public List<String> getAllDates(String startDate, String endDate) {
        List<String> result = new ArrayList<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate start = LocalDate.parse(startDate, formatter);
        LocalDate end = LocalDate.parse(endDate, formatter);
        while (!start.isAfter(end)) {
            result.add(start.format(formatter));
            start = start.plusDays(1);
        }
        return result;
    }
}
